﻿--[[
	LootMaster player/candidate Stuff
	EDIT BY RS
	See line 43
	Line 213 added ENCHSKILL command
]]--
local lm 	= LootMaster		-- Local instance of the addon

local L = LibStub("AceLocale-3.0"):GetLocale("CouncilLootMaster")

function LootMaster:Debug( message, verbose )
    if not self.debug then return end;
    if verbose and not self.verbose then return end;
    self:Print("debug: " .. message)
end

function LootMaster:SendCommand(command, message, target)
	if not target then
		return self:Print(L["Could not send command, no target specified"])
	end;

    local formatted = format("%s:%s", tostring(command), tostring(message));

	-- Speedup messages to self by just calling the ML CommandReceived function.
    if LootMasterML and target == UnitName("player") then
		LootMasterML.CommandReceived(LootMasterML, nil, formatted, "WHISPER", target)
    elseif LootMaster:UnitInRaid(target) then
        -- we're in raid with master looter
		self:SendCommMessage("CLootMasterML", formatted, "RAID", nil, "ALERT")
        self:Debug('SendCommand(RAID): '..formatted, true)
    elseif UnitInParty(target) and GetNumSubgroupMembers()>0 then
        --we're in party with master looter
        self:SendCommMessage("CLootMasterML", formatted, "PARTY", nil, "ALERT")
        self:Debug('SendCommand(PARTY): '..formatted, true)
    else
        --we're not grouped, send message to target by whispering
        self:SendCommMessage("CLootMasterML", formatted, "WHISPER", target, "ALERT")
        self:Debug('SendCommand(WHISPER->'..target..'): '..formatted, true)
    end    
end

--[[
	Event gets triggered when candidate receives a message from the Masterlooter
]]
function LootMaster:CommandReceived(prefix, message, distribution, sender)
	local _,_,command, message = string.find(message, "^([%a_]-):(.*)$")
	command = strupper(command or '');
	message = message or '';

	if command == 'DO_YOU_WANT' then

		-- Masterlooter wants to know from us if we'd like to have the item.
		-- Lets show the gui and ask the player for input.

		local itemID, ilevel, binding, slot, quality, timeout, link, texture, notesAllowed = strsplit("^", message)
		local playerLocalClass,playerClass = UnitClass("player")
		
		if not self:EligibleItemCheck( link, playerLocalClass, playerClass ) then
			self:Print(format(L['Autopassing %s (not eligible)'], link or 'unknown item'))
			return
		end
		
		notesAllowed = ((tonumber(notesAllowed or 0) or 0) == 1)
		-- Send the master loot our current gear and version number
		self:SendCommand( 'GEAR', format('%s^%s', itemID, self:GetGearByINVTYPE(slot, 'player')), sender)

		if not self:HasLoot(link) then
			-- add the loot to the lootlist and redraw the ui
			tinsert( self.lootList, {
				["lootmaster"]      = sender,
				["link"]            = link,
				["id"]              = itemID,
				["notesAllowed"]    = notesAllowed,
				["ilevel"]          = ilevel,
				["binding"]         = binding,
				["slot"]            = slot,
				["texture"]         = texture,
				["timeout"]         = tonumber(timeout),
				["timeoutLeft"]     = tonumber(timeout),
				["quality"]         = tonumber(quality)
			})

			self.lootMLCache[link] = sender;

			self:UpdateLootUI();
		end

  elseif command == 'DISCARD' then

    -- Message gets received
    local itemID, link = strsplit("^", message)

    if self:RemoveLoot(link) then
        self:UpdateLootUI();
    end

  elseif command == 'LOOTED' then
    -- Someone looted an item through lootmaster.
    -- Update the UI and send some info to ct_raidtracker if its active.

    -- Message gets received
    local player, link, lootType = strsplit("^", message)

    self:Debug('looted: ' .. message)

    if self.lootMLCache[link] and self.lootMLCache[link] ~= sender then
        return self:Print(format('%s sent a message that %s has been looted, but is not lootmaster for this item (his version outdated?). Message ignored', sender, link))
    end
    self.lootMLCache[link] = nil

    if self:RemoveLoot(link) then
        self:UpdateLootUI()
    end

    self:Debug('UI updated')

    if not player or not link then
        return self:Debug('!player or !link')
    end;

    lootTypeID = tonumber(lootType) or LootMaster.LOOTTYPE.UNKNOWN
    lootType = LootMaster.LOOTTYPE[lootTypeID] or LootMaster.LOOTTYPE[LootMaster.LOOTTYPE.UNKNOWN]

  elseif command == 'ENCHSKILL' then
	
	-- masterlooter wants to know if we have sufficient enchanting skill
	if self:GetEnchantingSkill() >= tonumber(message) then
		self:SendCommand('ENCHRESP', true, sender)
	end;
  
  end;
end


function LootMaster:HasLoot( link )
    for i, data in ipairs(self.lootList) do repeat
        if not data then break end
        if data.link == link or data.id == link then
            return i
        end
    until true end
    return nil
end

function LootMaster:GetLoot( link )
    local index = self:HasLoot(link)
    if not index then return nil end
    return self.lootList[index]
end

function LootMaster:RemoveLoot( link )
    local index = self:HasLoot(link)
    if not index then return false end
    tremove(self.lootList, index)
    return true
end

function LootMaster:SendItemWanted( lootmaster, itemLink, response, note)
    -- Just whisper the response back to the ml.      FINDBUG
	local _, avgItemLevel = GetAverageItemLevel()
    self:SendCommand("WANT", format("%s^%s^%s^%s", itemLink or '', response or 0, floor(avgItemLevel + 0.5), gsub(note or '','%^','')), lootmaster)
end

--[[ Get the enchantingSkill as number (0 if not enchanter)
]]--
function LootMaster:GetEnchantingSkill()
	if _G.GetProfessions~=nil then
		local p1, p2 = GetProfessions()
		if p1==nil and p2==nill then return 0 end
		local enchNameLocalized = GetSpellInfo(7411) -- Enchanting - Apprentice
		if not enchNameLocalized then return 0 end
		if p1~=nil then
			local skillName, _, skillRank = GetProfessionInfo(p1)
			if skillName == enchNameLocalized then return skillRank end
		end
		if p2~=nil then
			local skillName, _, skillRank = GetProfessionInfo(p2)
			if skillName == enchNameLocalized then return skillRank end
		end
		return 0
	end

    local numSkills = GetNumSkillLines()
    local enchNameLocalized = GetSpellInfo(7411) -- Enchanting - Apprentice
    if not enchNameLocalized then return 0 end;
    for i=1, numSkills do
        local skillName, _, _, skillRank = GetSkillLineInfo(i)
        if skillName == enchNameLocalized then return skillRank end;
    end
    return 0
end

--[[
	Kind of singleton, just create the frame once.
]]
function LootMaster:GetLootFrame()
	if self.lootframe then return self.lootframe end
	return 'test'
end
